@@ -1,35 +1,16 @@
-# ============================================================================
-# The MIT License
-#
-# Copyright (c) 2018 Infineon Technologies AG
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE
-# ============================================================================
-from ctypes import *
+#!/usr/bin/env python
+"""This module defines backend operations and works as a gateway between the library and the actual hardware """
+
+from ctypes import c_ubyte, c_ushort, c_int, POINTER, cdll
 import os
 import platform
 import sys
-from serial.tools import list_ports
 from re import match
 
+from serial.tools import list_ports
 
-_optiga_cddl = None
+
+_OPTIGA_CDLL = None
 
 
 __all__ = [
@@ -103,9 +84,9 @@ def _load_lib(interface):
 
     try:
         api = cdll.LoadLibrary(lib_path)
-    except OSError:
+    except OSError as fail_to_load:
         os.chdir(old_path)
-        raise OSError('{}: Failed to find library {} in {}'.format(interface, libname, curr_path))
+        raise OSError('{}: Failed to find library {} in {}'.format(interface, libname, curr_path)) from fail_to_load
     api.exp_optiga_init.restype = c_int
     ret = api.exp_optiga_init()
     if ret != 0:
@@ -128,34 +109,92 @@ def _set_com_port_config(com_port):
             'opts is specified, but value parameter is given: expected COMXX, your provided {0}. '
             'Use set_com_port(\'COM39\')'.format(com_port)
         )
-    with open(ini_path, 'w', encoding='utf-8') as f:
-        f.write(com_port)
+    with open(ini_path, 'w', encoding='utf-8') as file:
+        file.write(com_port)
 
 
 def get_handler():
-    global _optiga_cddl
+    # pylint: disable=global-statement
+    # This is fair to use a global, as there should be only one instance of the communication stack initialized
+    """
+    A function which should return a communication instance of the connected hardware depending gon the interface
 
-    if _optiga_cddl is None:
+    """
+    global _OPTIGA_CDLL
+
+    if _OPTIGA_CDLL is None:
         supported_interfaces = ('libusb', 'uart', 'i2c')
         initialised = False
         errors = list()
-        """
-        Here we try to probe which interface is actually in use, might be either libusb, i2c or uart
-        We suppress stderr output of the libusb interface in case it's not connected to not confuse
-        a user
-        """
+        # Here we try to probe which interface is actually in use, might be either libusb, i2c or uart
+        # We suppress stderr output of the libusb interface in case it's not connected to not confuse
+        # a user
         for interface in supported_interfaces:
             try:
-                _optiga_cddl = _load_lib(interface)
+                _OPTIGA_CDLL = _load_lib(interface)
                 print('Loaded: {0}'.format(_get_lib_name(interface)))
                 initialised = True
+                break
             except OSError as error:
                 errors.append(error)
-                pass
 
         if not initialised:
             for err in errors:
                 print(err)
-            exit()
+            sys.exit()
 
-    return _optiga_cddl
+    return _OPTIGA_CDLL
+
+
+def protected_update(api, manifest, fragments):
+    # pylint: disable=global-statement
+    # This is fair to use a global, as there should be only one instance of the communication stack initialized
+    """
+    A function which should return a communication instance of the connected hardware depending gon the interface
+
+    :param api: api hardware handler
+
+    :param manifest: a byte object containing a manifest data to be sent to the chip
+
+    :param fragments: a list of individual bytes objects containing a payload to be sent to the chip
+    """
+    _manifest = (c_ubyte * len(manifest))(*manifest)
+
+    api.exp_optiga_util_protected_update_start.argtypes = c_ubyte, POINTER(c_ubyte), c_ushort
+    api.exp_optiga_util_protected_update_start.restype = c_int
+
+    ret = api.exp_optiga_util_protected_update_start(c_ubyte(0x01), _manifest, len(_manifest))
+
+    if ret != 0:
+        print("Manifest [{0}]: ".format(len(_manifest)).join('{:02x} '.format(x) for x in list(_manifest)))
+        print(len(_manifest))
+        raise IOError('Function can\'t be executed. Error {0}'.format(hex(ret)))
+
+    for count, fragment in enumerate(fragments[:-1]):
+        _fragment = (c_ubyte * len(fragment))(*fragment)
+
+        api.exp_optiga_util_protected_update_continue.argtypes = POINTER(c_ubyte), c_ushort
+        api.exp_optiga_util_protected_update_continue.restype = c_int
+
+        ret = api.exp_optiga_util_protected_update_continue(_fragment, len(_fragment))
+
+        if ret != 0:
+            print("Fragment {0} [{1}]: ".
+                  format(count, len(_fragment)).
+                  join('{:02x} '.format(x) for x in list(_fragment)))
+            raise IOError('Function can\'t be executed. Error {0}'.format(hex(ret)))
+
+    final_fragment = (c_ubyte * len(fragments[-1]))(*fragments[-1])
+
+    api.exp_optiga_util_protected_update_final.argtypes = POINTER(c_ubyte), c_ushort
+    api.exp_optiga_util_protected_update_final.restype = c_int
+
+    ret = api.exp_optiga_util_protected_update_final(final_fragment, len(final_fragment))
+
+    if ret != 0:
+        print("Final Fragment [{0}]: ".
+              format(len(final_fragment)).
+              join('{:02x} '.format(x) for x in list(final_fragment)))
+        raise IOError('Function can\'t be executed. Error {0}'.format(hex(ret)))
+
+    return ret
